require "log"
require "math"
require "globals"
require "Fighter"
require "helpers"

AIStates = {
	TURN_STANDARD = 6,
	TURN_FINE = 5,
	TURN_AGGRESSIVE = 4,
	TURN_HOLD = .45,
	MOVE_FORWARD_STANDARD = 2,
	MOVE_FORWARD = 3,
	NONE = -1,
	SLEEP = 0
}

-- fighter class
AI = inheritsFrom(nil)
			--Object definition


function AI:new(world,fighter,name)
	self.state = AIStates.NONE --standard, hold, aggressive, fine

	self.heading = 0

	move_moving = false
	move_forwardFor = 0
	move_waitUntil = 0
	move_reverseUntil = 0
	move_dtSum = 0

	turn_turning = false
	turn_toHeading = 0
	turn_turnDirection = 1
	turn_turnUntil = 0
	turn_waitUntil = 0
	turn_reverseUntil = 0
	turn_dtSum = 0

	startMoves = true
	continueMoves = true

	sleepTo = 0

	self.name = name or "John Doe"
	self.actionQ = {}
	self.world = world
	self.fighter = fighter
	self.fighter:setPilot(self)

	return self
end

function AI:q(func, expedite)
	print("Qing: " ..  func[1])
	if expedite then
		table.insert(self.actionQ, 1, func)
	else
		self.actionQ[#self.actionQ+1] = func
	end
end


function AI:popQ()

	local action = self.actionQ[1]
	print("PopQ " .. action[1])
	if not action then print("Nothing shaken") ; return end
	table.remove(self.actionQ,1) --Pop
	
	--local param1, param2, param3, param4, 	
	--Level 1 Actions- movement
	if self.startMoves then
		if action[1] == "turnToHeading" then self:turnToHeading(action[2], action[3]);return
		elseif action[1] == "forwardThenFullStop" then self:forwardThenFullStop(action[2], action[3]);return
		elseif action[1] == "forward" then self:forward(action[2], action[3]);return
		end
	end

	--Level 2 Actions (call level 1 actions)
	if action[1] == "turnToObject" then self:turnToObject(action[2]);return
	elseif action[1] == "goToXYFullStop" then self:goToXYFullStop(action[2], action[3], action[4]);return
	elseif action[1] == "returnToAO" then self:returnToAO();return
	elseif action[1] == "stop" then self:stop();return

	--Auxilliary actions
	elseif action[1] == "giveControlTo" then self:giveControlTo(action[2]);return
	elseif action[1] == "sleep" then self:sleep(action[2]) end
end

function AI:sleep(seconds)
	if seconds then --Setting the sleep timer
		self.sleepTo = love.timer.getTime() + seconds 
		self.state = AIStates.SLEEP
	elseif love.timer.getTime() >= self.sleepTo then 
		self.sleepTo = 0
		self.state = AIStates.NONE
	end		
end

function AI:goToXYFullStop(x, y, reverse_ok)
	local adjusted_x = x - self.fighter.xVel
	local adjusted_y = y - self.fighter.yVel
	print ("GoToXYFullStop: " .. x .. " " .. y)
	self:q( {"forwardThenFullStop", x, y, reverse_ok}, true )
	self:q( {"sleep", .5}, true )
	self:q( {"turnToHeading", XYToAngle(self.fighter.x, self.fighter.y, adjusted_x, adjusted_y), reverse_ok}, true)

end

--if inverse_ok then we can turn directly away from track if its closer
function AI:turnToTrack(inverse_ok)
	local track = math.atan(self.fighter.yVel/self.fighter.xVel)
	local relative_track = rationalizeRad(track-self.heading)
	if not inverse_ok or (relative_track >= 4.72 and relative_track <= 1.57) then
		self:q( {"turnToHeading", track}, true )
		return true --We're turning towards track
	else
		--turn away from movement
		self:q( {"turnToHeading", rationalizeRad(track+math.pi)}, true )
		return false --We're turning away from track
	end
end

function AI:stop()
	AI:goToXYFullStop(self.fighter.x, self.fighter.y, true)	
end

function AI:forward(x, y) --towards x and y
	local adjusted_x = x - self.fighter.xVel
	local adjusted_y = y - self.fighter.yVel
	d = distanceToXY(self.fighter.x, self.fighter.y, adjusted_x, adjusted_y)
	d = d/3 --forward burn only first third
	self.move_dtSum = 0
	local a = self.fighter.thrust * 2 *10
	self.move_forwardFor = math.sqrt(2*d/(a))
	--self.turnFor = math.sqrt(2*(relative_turn_to/3) / .03)	
	self.move_waitUntil = self.move_forwardFor * 1.5
	--Of course reverseUntil won't activate a reverse, but will let us know when the move is done
	self.move_reverseUntil = self.move_waitUntil + self.move_forwardFor
	self.state = AIStates.MOVE_FORWARD
	
end

--should already be facing our target
function AI:forwardThenFullStop(x,y)
	print("Targets: " .. x .. " " .. y )
	local adjusted_x = x - self.fighter.xVel
	local adjusted_y = y - self.fighter.yVel
	d = distanceToXY(self.fighter.x, self.fighter.y, adjusted_x, adjusted_y)
	self.move_dtSum = 0
	print("Distance: " .. d)
	d = d/3 --forward burn only first third
	local a = self.fighter.thrust * 2 *10
	self.move_forwardFor = math.sqrt(2*d/(a))
	--self.turnFor = math.sqrt(2*(relative_turn_to/3) / .03)	
	self.move_waitUntil = self.move_forwardFor * 1.5
	self.move_reverseUntil = self.move_waitUntil + self.move_forwardFor
	self.state = AIStates.MOVE_FORWARD_STANDARD
end

function AI:turnToObject(worldObject, rate)
	self:turnToHeading(XYToAngle(self.fighter.x, self.fighter.y, worldObject.x, worldObject.y), rate)	
end

function AI:turnToHeading(heading, rate)
	self.turn_dtSum = 0
	self.turn_toHeading = heading
	local relative_turn_to = self.turn_toHeading - rationalizeRad(self.heading)

	relative_turn_to = rationalizeRad(relative_turn_to)

	if relative_turn_to < .02 and math.abs(self.fighter.rVel) < .02 then
		self.state = AIStates.NONE
		return
	end
	relative_turn_to = rationalizeRad(relative_turn_to - self.fighter.rVel)

	if relative_turn_to <= math.pi then --Target right
		self.turn_turnDirection = 1		
	else
		self.turn_turnDirection = -1
		relative_turn_to = 2*math.pi - relative_turn_to
	end
--	print ("After left/right consideration: " .. relative_turn_to .. " and " .. self.turn_turnDirection)
	--1.2 Radians per 1 second of burn and 1 second of wait

	--2 (just part of the formula) * theta/3(burn only a third) / 1(acceleration)
	--Acceleration: .005 * 2Thrusters * EngineStatus(one to ten... ten..)
	self.turn_turnFor = math.sqrt(2*(relative_turn_to/3) / 1)	
	self.turn_waitUntil = (self.turn_turnFor * 1.5) - self.fighter.rVel --cut into wait time for reverse
	self.turn_reverseUntil = self.turn_waitUntil + self.turn_turnFor + self.fighter.rVel
	print ("Turning for: " .. self.turn_turnFor)
	self.state = AIStates.TURN_STANDARD
end

function AI:update(dt)
	if not self.fighter then return end
	self.heading = self.fighter.heading
--print ("updating " .. tostring(self.state)±)
	self:energyManager()
	if self.state == AIStates.NONE and self.actionQ[1] ~= nil then
		self:popQ()
	end
	if self.continueMoves then
		if self.state == AIStates.TURN_STANDARD then self:turnUpdate2(dt)
		elseif self.state == AIStates.TURN_AGGRESSIVE then self:turnUpdate2(dt) 
		elseif self.state == AIStates.TURN_FINE then self:turnUpdate(dt)
		elseif self.state == AIStates.MOVE_FORWARD_STANDARD or
		 self.state == AIStates.MOVE_FORWARD then self:moveUpdate(dt) 
		end

	end

	if self.state == AIStates.SLEEP then self:sleep() end 
	
end


function AI:moveUpdate(dt)

	local burn_forward = false
	local burn_reverse = false
	if self.move_dtSum  < self.move_forwardFor then
		burn_forward = true
	--print("forward")
	elseif self.move_dtSum >= self.move_waitUntil and self.move_dtSum < self.move_reverseUntil then
		if self.state == AIStates.MOVE_FORWARD_STANDARD then		
			burn_reverse = true
		--print("reverse")
		end
	
	elseif self.move_dtSum >= self.move_reverseUntil then
		--print ("done")
		print ("FINAL: X: " .. self.fighter.x .. " Y: " .. self.fighter.y)
		self.state = AIStates.NONE
	end
	self.move_dtSum = self.move_dtSum + dt
	--print(self.move_dtSum)
	if burn_forward then self.fighter:burnEngine(3, dt) self.fighter:burnEngine(4, dt) 
	elseif burn_reverse then self.fighter:burnEngine(1, dt) self.fighter:burnEngine(2, dt) end
end


function AI:turnUpdate(dt)
	print ("Turning fine")
	local relative_turn_to = self.turn_toHeading - self.heading
	relative_turn_to = rationalizeRad(relative_turn_to)
	
	local reverse_at = self.turn_toHeading - self.fighter.rVel 

	if relative_turn_to < .1 and math.abs(self.fighter.rVel) < .1 then
		self.state = AIStates.NONE
		return
	end

	local target_right = false
	local burn_right = false
	local burn_left = false
	target_right = relative_turn_to < math.pi

	if target_right then
--print("Target right")
		if self.fighter.rVel < relative_turn_to/6 then burn_right = true end
		if self.heading > reverse_at then 
			burn_right = false
			burn_left = true
--			print ("reverse burn for target right")
		end
	else
--print("Target left")
		if self.fighter.rVel > -relative_turn_to/6 then burn_left = true end
		if self.heading < reverse_at then
			burn_right = true
			burn_left = false
--print ("reverse burn for target left")
		end
	end

	if burn_right then self.fighter:burnEngine(2, dt) self.fighter:burnEngine(4, dt) 
	elseif burn_left then self.fighter:burnEngine(1, dt) self.fighter:burnEngine(3, dt) end
end

function AI:turnUpdate2(dt)
	--print ("Turn Update: " .. self.turn_turnDirection .. " sum: " .. self.turn_dtSum)
	local burn_right = false
	local burn_left = false
--one frame of burning is 0.0199999 rVel
	if self.turn_dtSum < self.turn_turnFor then
		--not done burn stage
		if self.turn_turnDirection > 0 then burn_right = true else burn_left = true end
				
	elseif self.turn_dtSum >= self.turn_waitUntil and self.turn_dtSum < self.turn_reverseUntil then
		--reverse burn stage
		if self.turn_turnDirection > 0 then burn_left = true else burn_right = true end

	elseif self.turn_dtSum >= self.turn_reverseUntil then
		print ("Final Heading: " .. self.heading .. " (Target was: " .. self.turn_toHeading .. ")")
		print ("Difference: " .. math.abs(self.turn_toHeading-self.heading))
		print ("rVel: " .. self.fighter.rVel)
		print ("Difference in times: " .. self.turn_dtSum - self.turn_reverseUntil)
		if math.abs(self.fighter.rVel) > .02 or
		 	(math.abs(self.turn_toHeading-self.heading) > 0.02 and math.abs(self.turn_toHeading-self.heading) < (2*math.pi-.02))  then
			print ("Correcting.")			
			--self:q({"turnToHeading", self.turn_toHeading}, true)
		end
		self.state = AIStates.NONE
	end
	self.turn_dtSum = self.turn_dtSum + dt

	if burn_right then self.fighter:burnEngine(2, dt) self.fighter:burnEngine(4, dt) 
	elseif burn_left then self.fighter:burnEngine(1, dt) self.fighter:burnEngine(3, dt) end
end

function AI:turnUpdate3(dt)
	--From rotational kinematics:
	--This is the heading at which, given the current rVel, we will need to burn at to 
	--reach
	local burn_right = false
	local burn_left = false 
	local reverse_at = ((self.fighter.rVel * self.fighter.rVel) + 2*self.turn_toHeading)/ 2
	--Now it's relative!
	reverse_at = rationalizeRad(reverse_at - self.heading)
	print ("TO: " .. self.turn_toHeading .. " Heading: " .. self.heading .. " dif: " .. math.abs(self.turn_toHeading-self.heading))
	if math.abs(self.turn_toHeading-self.heading) < .1 and math.abs(self.fighter.rVel) < .1 then
		self.state = AIStates.NONE
		return
	end
	print ("reverse at: " .. reverse_at)
	if reverse_at <= math.pi and reverse_at >= 0 then 
		--Reverse burn point is right
		--we're turning left, so we've past it:
		print("reverse burning")
		if self.turn_turnDirection == 0 then burn_right = true end
	else --Reverse point is left, and we're turning right so we've past it
		print ("reverse burning 2")
		if self.turn_turnDirection == 1 then burn_left = true end
	end

	if not burn_right and not burn_left then
		if self.turn_dtSum < self.turn_turnFor then
			--not done burn stage
			if self.turn_turnDirection > 0 then burn_right = true else burn_left = true end
		end
	end
	self.turn_dtSum = self.turn_dtSum + dt

	if burn_right then self.fighter:burnEngine(2, dt) self.fighter:burnEngine(4, dt) 
	elseif burn_left then self.fighter:burnEngine(1, dt) self.fighter:burnEngine(3, dt) end
end

function AI:returnToAO()
	local adjusted_x = 0 - self.fighter.xVel
	local adjusted_y = 0 - self.fighter.yVel
	self:q( {"forward", 0, 0}, true )

	self:q( {"turnToHeading", XYToAngle(self.fighter.x, self.fighter.y, adjusted_x, adjusted_y)}, true, true )

end

function AI:giveControlTo(pilot)
	self.fighter.pilot = pilot
	self.fighter = nil
end

--Returns > 0 if 
function AI:energyManager()
	if self.fighter.energy > 10 then 
		self.startMoves = true
		self.continueMoves = true
	elseif self.fighter.energy < 1 then
		self.continueMoves = false --Gonna start burning just one burner...
	else
--		self:q({"sleep", 2})
		self.startMoves = false
	end
	
end
