require "log"
require "math"
require "globals"
require "WorldObject"

Asteroid = inheritsFrom(WorldObject)


function Asteroid:new(x,y, xVel, yVel, rVel, heading, number)
	local self = self:create()
	self.image_map = nil
	self.quad = nil
	self.r = 50
	self.mass = 100
	self.scale = 1
	self.x,self.y = x, y
	self.xVel = xVel or 0
	self.yVel = yVel or 0
	self.heading = heading or 0
	self.rVel = rVel or 0
	self.number = number or 1
	self.name = "Asteroid(" .. number .. ")" 
	if self.number == 1 then
		self.w =  66
		self.h = 106
		self.offset_x = 0
		self.offset_y = 0
	elseif self.number == 2 then
		self.w =  55
		self.h = 45
		self.offset_x = 66
		self.offset_y = 3
	elseif self.number == 3 then
		self.w = 52
		self.h = 60
		self.offset_x = 66
		self.offset_y = 53
	elseif self.number == 4 then
		self.w = 74
		self.h = 104
		self.offset_x = 126
		self.offset_y = 3
		
	end 

	self:loadSprite()	--Loads the sprite into the above vars

	self:defineHitCircles()	 
	self.status = 1
	return self
end

function Asteroid:getX() return self.x end
function Asteroid:getY() return self.y end
function Asteroid:getXYWH() --for camera
	return { self.x, self.y, self.w, self.h }
end

function Asteroid:getCircles() --for collider
	--Rotation around the center of the object:
	--x' = x cos f - y sin f
	--y' = y cos f + x sin f
	local circle_list = {}
	--go thru hit circles, rotate them around the center, add the object's X,Y, package them
	--in a new table, return it
	for i,circle in ipairs(self.hitCircles) do
		local temp_circle = {}
		temp_circle.x = self.x+((circle.x*math.cos(self.heading)) + (circle.y*math.sin(self.heading)))
		temp_circle.y = self.y+((circle.y*math.cos(self.heading)) - (circle.x*math.sin(self.heading)))
		temp_circle.r = circle.r
		circle_list[#circle_list+1] = temp_circle --Add to temp circle listing
	end
	return circle_list
end

--Set up the fighter's hit circles (relative center of object)
function Asteroid:defineHitCircles()
	local circle_list = {}
	if self.number == 1 then
		local top = {}
			top.x = 0
			top.y = 25
			top.r = 30
		local bottom = {}
			bottom.x = 0
			bottom.y = -25
			bottom.r = 30
		circle_list[1] = top
		circle_list[2] = bottom
		self.boundingRadius = 53
	elseif self.number == 2 then
		local top = {}
			top.x = 0
			top.y = 0
			top.r = 26
		circle_list[1] = top
		self.boundingRadius = 26		
	elseif self.number == 3 then
		local top = {}
			top.x = 0
			top.y = 0
			top.r = 27
		circle_list[1] = top
		self.boundingRadius = 27	
	elseif self.number == 4 then
		local top = {}
			top.x = 0
			top.y = 25
			top.r = 30
		local bottom = {}
			bottom.x = 0
			bottom.y = -25
			bottom.r = 30
		circle_list[1] = top
		circle_list[2] = bottom
		self.boundingRadius = 102
	end
	--print ("Circle list 1: " .. circle_list[1])
	self.hitCircles = circle_list
end

function Asteroid.preLoad()
	global.asteroidImage = love.graphics.newImage(global.asteroidImageName)
end

function Asteroid:loadSprite()
	self.image_map = global.asteroidImage
    local image_map_w, image_map_h = self.image_map:getWidth(), self.image_map:getHeight()

    local asteroid_sprite_w, asteroid_sprite_h = self.w, self.h
	self.quad = love.graphics.newQuad(self.offset_x, self.offset_y, asteroid_sprite_w, asteroid_sprite_h,
											image_map_w, image_map_h)

	if self.image_map == nil then log("*Error loading Asteroid image.")
		else log(" Done loading Asteroid image.") end
end

function Asteroid:draw()
	--print (self.number)
	--Move to top left of the object for drawing
	love.graphics.rotate(self.heading)
	love.graphics.translate(-self.w/2,-self.h/2) 

	love.graphics.scale(self.scale)
	--Draw the Asteroid
	love.graphics.drawq(self.image_map, self.quad, 0, 0)	
end

function Asteroid:update(dt)
	
	self.x = self.x + self.xVel*dt
	self.y = self.y + self.yVel*dt
	self.heading = self.heading + self.rVel*dt
end

function Asteroid:receiveCollision(hit_list, world_object)


end


