
require "globals"
camera = {}
camera.x, camera.y = 0, 0
camera.width, camera.height = 1024,768 --Screen size
camera.scaleX, camera.scaleY = .3, .3		--scale (bigger for higher resolutions
camera.buffer1	= love.graphics.newFramebuffer(1024, 768)
camera.buffer2 = love.graphics.newFramebuffer(1024, 768)
camera.bufferSwitch = -1
camera.screenCircle = nil

function camera.set(width, height)
	camera.width = width or 1024
	camera.height = height or 768
	love.graphics.setMode(camera.width,camera.height, false)

	camera.buffer1	= love.graphics.newFramebuffer(camera.width, camera.height)
	camera.buffer2  = love.graphics.newFramebuffer(camera.width, camera.height)
	love.graphics.setColorMode("replace")
	--love.graphics.setBlendMode("additive")
	camera.screenCircle = {}
	camera.screenCircle.r = global.screenRadius --2200 pixels of the world from the player will always be drawn

	camera.scaleX = camera.width*.4 / 1024
	camera.scaleY = camera.width*.4 / 1024
end
function camera.setScale(new_scale)
	camera.scaleX = new_scale
	camera.scaleY = new_scale
	camera.screenCircle.r = (camera.width / 2)/camera.scaleX + 500
end

--Translate and scale from Love's default screen to the world's system
function camera.translateToWorld(x,y,heading)
	-- Draw from the middle of the screen
	love.graphics.translate(camera.width/2, camera.height/2)
	love.graphics.rotate(-heading)
	love.graphics.scale(camera.scaleX,camera.scaleY)
	love.graphics.translate(-x, y)
end
--x,y is where the player is
--width and height are the screen size
function camera.draw(world, x, y, heading)

	camera.bufferSwitch = camera.bufferSwitch*-1
	if camera.bufferSwitch > 0 then love.graphics.setRenderTarget(camera.buffer1)
	else love.graphics.setRenderTarget(camera.buffer2) end

	--local upper_left_x = x - (camera.width / 2) 
	--local upper_left_y = y - (camera.height / 2)
	local object_list = nil
	camera.screenCircle.x = x
	camera.screenCircle.y = y

	love.graphics.push()
	camera.translateToWorld(x, y, heading)


	local obj_circle = {}

	camera.drawBackground(world.background, world.backgroundQuad, x, y, heading)

	--Grab each of the lists (objects, explosions). Go through each, draw each object if it's on
	--screen.
	for k, object_list in ipairs(world:getDrawLists()) do
		for i,world_object in ipairs(object_list) do
			if world_object == nil then
				log("Error: nil object found in world object list", 9)		
				break
			elseif not world_object.visible then break	
			end
			obj_circle.x = world_object.x
			obj_circle.y = world_object.y
			obj_circle.r = world_object.boundingRadius				 
			--Make sure the object is on the screen, otherwise it doesnt get a chance to draw
			--if boundingRadius = -1 then it's always drawn
			if world_object.boundingRadius <= 0 or check2Circles(camera.screenCircle, obj_circle) then

				--Move to the player's fighter, from there, move to the object to draw, then
				--move to the left a bit so we dont draw the object off to the right and down
				love.graphics.push()
					love.graphics.translate(obj_circle.x, -obj_circle.y)
					world_object:draw(heading)

				love.graphics.pop()	
			end	
		end
	end

	if global.showHitBoxes then
		for k, object_list in ipairs(world:getDrawLists()) do
			for i,world_object in ipairs(object_list) do
				if world_object == nil then
					log("Error: nil object found in world object list", 9)		
				 break end
				for m,circle in ipairs(world_object:getCircles()) do
					love.graphics.push()
					draw_x = circle.x -- x
					draw_y = circle.y -- y
					love.graphics.translate(draw_x, -draw_y)				
					love.graphics.circle("line",0,0, circle.r)
					love.graphics.pop()
				end	
			end
		end		
	end
	love.graphics.pop()
	--TLfres.letterbox()
	love.graphics.setRenderTarget( )
	if camera.bufferSwitch > 0 then love.graphics.draw(camera.buffer1,0,0)
	else love.graphics.draw(camera.buffer2,0,0) end


end 

function camera.drawBackground(bg, quad,x,y,h)
	love.graphics.push()
	love.graphics.translate(x, -y)
	love.graphics.scale(1/camera.scaleX,1/camera.scaleY)
	quad:setViewport(0,0,camera.width*2, camera.height*2)
	love.graphics.translate(-camera.width*2, -camera.height*2)
	love.graphics.drawq( bg, quad, -(.3*x)*camera.scaleX*3, (.3*y)*camera.scaleY*3, 0, 1.5, 1.5, 0,0 )
	love.graphics.pop()
end
