require "log"
require "math"
require "globals"


Explosion = inheritsFrom(WorldObject) 


function Explosion:new(x,y, xVel, yVel)
	local self = self:create()
	self.name = "Explosion A"
	self.image_map = nil
	self.explosion_quad = nil
	self.shockwave_map = nil
	self.shockwave_quad = nil
	self.x = x
	self.y = y
	self.xVel = 0
	self.yVel = 0
	self.w = 64
	self.h = 64
	self.r = 10
	self.offset_x = 0
	self.offset_y = 0
	self.timer = 0
	self.shockwaveTimer = 0
	self.scale = 1.5
	self.status = 1
	self:loadSprite()
	return self
end

function Explosion.preLoad()
	global.explosionImage = love.graphics.newImage(global.explosionImageName)
	global.shockwaveImage = love.graphics.newImage(global.shockwaveImageName)
end
function Explosion:loadSprite()
	self.image_map = global.explosionImage
    local image_map_w, image_map_h = self.image_map:getWidth(), self.image_map:getHeight()

    local explosion_sprite_w, explosion_sprite_h = self.w, self.h
	self.explosion_quad = love.graphics.newQuad(0, 0, explosion_sprite_w, explosion_sprite_h,
											image_map_w, image_map_h)
	self.shockwave_map = global.shockwaveImage
	self.shockwave_quad = love.graphics.newQuad(0, 0, 128,128,128,128)
end

function Explosion:draw()
	love.graphics.push()
	love.graphics.setColorMode("modulate")	
	love.graphics.setColor(255,255,255,210-(self.shockwaveTimer*30))
	love.graphics.translate(-self.shockwaveTimer*64, -self.shockwaveTimer*64) 
	love.graphics.drawq(self.shockwave_map, self.shockwave_quad, 0, 0, 0, self.shockwaveTimer, self.shockwaveTimer)
	love.graphics.pop()
	love.graphics.setColorMode("replace")
	--Move to top left of the object for drawing
	love.graphics.translate(-self.w/2,-self.h/2) 
	love.graphics.drawq(self.image_map, self.explosion_quad, 0, 0, 0)	
end

function Explosion:update(dt)
	self.x = self.x + self.xVel*dt
	self.y = self.y + self.yVel*dt

	self.shockwaveTimer = self.shockwaveTimer + dt*2
	self.timer = self.timer + dt
	if self.timer > 0.2 then 

		self.offset_x = (self.offset_x + self.w) % (4*self.w)
		if self.offset_x == 0 then self.offset_y = self.offset_y + self.h end
		self.explosion_quad:setViewport(self.offset_x, self.offset_y, self.w, self.h)
		self.timer = 0
		if self.offset_y > self.h * 3 then self.status = 0 end
	end
end

