require "log"
require "math"
require "globals"
require "WorldObject"
require "Missile"

ExplosionB = inheritsFrom(WorldObject) 


function ExplosionB:new(x,y, xVel, yVel)
	local self = self:create()
	self.name = "Explosion B"
	self.x = x
	self.y = y
	self.image_map = global.explosionBImage
	self.explosion_quad = nil
	self.w = 256
	self.h = 256
	self.timer = 0
	self.scale = 1
	self.shockwave_map = nil
	self.shockwave_quad = nil
	self.shockwaveTimer = 0
	self:loadSprite()
	self.status = 1
	self.opacity = 150
	self.collides = false
	return self
end

function ExplosionB.preLoad()
	if not global.shockwaveImage then
		global.shockwaveImage = love.graphics.newImage(global.shockwaveImageName)
	end
	global.explosionBImage = love.graphics.newImage(global.explosionBImageName)
end

function ExplosionB:loadSprite()
	self.image_map = global.explosionBImage
    local image_map_w, image_map_h = self.image_map:getWidth(), self.image_map:getHeight()

    local explosion_sprite_w, explosion_sprite_h = self.w, self.h
	self.explosion_quad = love.graphics.newQuad(0, 0, explosion_sprite_w, explosion_sprite_h,
											image_map_w, image_map_h)
	self.shockwave_map = global.shockwaveImage
	self.shockwave_quad = love.graphics.newQuad(0, 0, 128,128,128,128)
end

function ExplosionB:extraDamage(world_object)
	return 10000 / distanceToXY(self.x, self.y, world_object.x, world_object.y)
end

function ExplosionB:draw()

	love.graphics.push()
	love.graphics.setColorMode("modulate")	
	love.graphics.setColor(255,255,255,255-(self.shockwaveTimer*30))
	love.graphics.translate(-self.shockwaveTimer*128, -self.shockwaveTimer*128) 
	love.graphics.drawq(self.shockwave_map, self.shockwave_quad, 0, 0, 0, self.shockwaveTimer*2, self.shockwaveTimer*2)
	love.graphics.pop()

	love.graphics.push()	
	love.graphics.setColor(255,255,255,self.opacity)
	love.graphics.setColorMode("modulate")	
	love.graphics.setBlendMode("additive")
	love.graphics.translate(-self.w/2,-self.h/2) 
	love.graphics.drawq(self.image_map, self.explosion_quad, 0, 0, 0, self.scale, self.scale)	
	love.graphics.pop()

	love.graphics.push()
	love.graphics.setColor(255,255,255,self.opacity*3)
	love.graphics.translate(-self.w/6,-self.h/6) 
	love.graphics.drawq(self.image_map, self.explosion_quad, 0, 0, 0, self.scale/3, self.scale/3)	
	love.graphics.setBlendMode("alpha")
	love.graphics.setColor(255,255,255,255)	
	love.graphics.pop()

end

function ExplosionB:update(dt)
	self.x = self.x + self.xVel*dt
	self.y = self.y + self.yVel*dt
	
	if self.timer < 0.4 then
		self.scale = self.scale + self.timer + self.timer
	elseif self.timer < 4 then
		self.opacity = self.opacity - dt*40
		self.scale = self.scale + self.timer
	else
		--Let it update once with collisions on. The next time thru after that, we're done
		if not self.collides then
			self.collides = true
		else self.status = 0
		end
	end
	self.w, self.h = 256*self.scale,256*self.scale
	self.boundingRadius = self.w / 2
	
	self.timer = self.timer + dt
	self.shockwaveTimer = self.shockwaveTimer + dt*2
	if self.opacity <0 then self.status = 0 end
end
