require "math"
hud = {
	fighter = nil,
	world = nil,
	width,
	height,
	timer,
	image_map,
	weap_quad,
	scanner
}

function hud.set(fighter, world)
	hud.fighter = fighter
	hud.world = world
	hud.width = love.graphics.getWidth()
	hud.height = love.graphics.getHeight()
	hud.timer = 0
end
function hud.preLoad()
	global.hudWeapImage = love.graphics.newImage(global.hudWeapImageName)

	hud.image_map = global.hudWeapImage
    local image_map_w, image_map_h = hud.image_map:getWidth(), hud.image_map:getHeight()

    local hud_weap_sprite_w, hud_weap_sprite_h = 256,128
	hud.weap_quad = love.graphics.newQuad(0, 0, hud_weap_sprite_w, hud_weap_sprite_h,
											image_map_w, image_map_h)

	if hud.image_map == nil then log("*Error loading hud image.")
		else log(" Done loading hud image.") end

end

function hud.draw()
	if not hud.fighter or hud.fighter.status < 1 or not hud.world then return end
	
	if love.timer.getTime()-hud.timer > .05 then
		hud.scanner = hud.scan()
		hud.timer = love.timer.getTime()
	end



	local heading = tonumber(string.format("%.2f", hud.fighter.heading*180/math.pi))
	local fps = tonumber(string.format("%.2f", love.timer.getFPS()))
	local time_since = tonumber(string.format("%.2f",love.timer.getTime()-hud.timer))
	love.graphics.push()
	
	love.graphics.setColorMode("modulate")

	love.graphics.push()
	--Translate to center of world (Fighter's X/Y are relative to center)

	camera.translateToWorld(hud.fighter.x,hud.fighter.y,hud.fighter.heading)

	hud.targetingComputer()

	-- World Boundary
	love.graphics.setColor(255,127,0)
	love.graphics.circle("line", 0, 0, 2800, 50)
	love.graphics.setColor(255,0,0)
	love.graphics.circle("line", 0, 0, hud.world.radius, 50)

	love.graphics.pop()
	--Energy bar
	--love.graphics.setColor(10,120,50)



	love.graphics.setLineWidth(3)
	love.graphics.translate(hud.width/2, hud.height/2)

	--Velocity cross
	love.graphics.push()
		love.graphics.rotate(-hud.fighter.heading)
		love.graphics.translate(-hud.fighter.xVel,hud.fighter.yVel)

		love.graphics.rotate(3.1415)
		love.graphics.print("- V -", -12, -12)
		--love.graphics.line(-10, 0, 10, 0)
		--love.graphics.line(0, 10, 0, 0)
	love.graphics.pop()


	love.graphics.setLineStipple(0xC000, 15)
--	love.graphics.line(0, 100,-300,-hud.height) --Electronics triangle
--	love.graphics.line(0, 100, 300,-hud.height)

	love.graphics.setLineWidth(1)
	love.graphics.line(0, 0, 0, -hud.height) --Aiming line
	love.graphics.setLineStipple(0xFC00, 8)
	--love.graphics.circle("line", 0, 0, 100, 100) --Missle activation circle
	love.graphics.setLineStipple(0xFFFF, 1)

--	love.graphics.print(heading, 100, -100, 0)
	love.graphics.print("FPS: " .. fps, 100, -80, 0)
--	love.graphics.print("Since: " .. time_since, 100, -70, 0)

	love.graphics.setColor(30,40,40,150)
	--love.graphics.translate(0, hud.height/2-40)
	hud.drawScan(hud.scanner)
	love.graphics.setColorMode("replace")
	love.graphics.pop()
	

	--love.graphics.drawq(hud.image_map, hud.weap_quad, hud.width-256, 20, 0)
	love.graphics.push()
		love.graphics.translate(hud.width-180, 20)
		love.graphics.setLineWidth(1)
		love.graphics.setLineStyle("smooth")

		love.graphics.setColor(21,24,41,110) --blue
		love.graphics.rectangle("fill", 0, 0, 180, 3)

		love.graphics.setColor(255,114,0) --orangle
		love.graphics.rectangle("fill", 180, 0, -180*(hud.fighter.energy/100), 3)

		love.graphics.setColor(255,255,255) --white
		love.graphics.line(180,0,180,3)

		love.graphics.translate(120, 100)
		love.graphics.setColor(21,24,41,110) --blue
		love.graphics.rectangle("fill",0,0,60,40)
		love.graphics.setColor(255,255,255) --white
		love.graphics.line(59,0,59,40)

	love.graphics.pop()
end

function hud.scan()
	if not hud.fighter or not hud.fighter.detectedObjects then return end
	hud.fighter:scan()
	--For each object with an energy signature
		--get the object's angle from me
		--add a value of the signature to the angle's key
	--For each degree in a circle around me
		--find the point in the arc
		--alter it based on the table above
		--draw a triangle to the point (base along the circle)
	local bearing
	local readings = {}
	local readings2 = {}
	local item = nil
	--Item is a scanner "reading"
	--{ world_object, emission, distance }
	for i,item in pairs(hud.fighter.detectedObjects) do
		
			bearing = XYToAngle(hud.fighter.x, hud.fighter.y, item[1].x, item[1].y)-hud.fighter.heading
			bearing = rationalizeRad(bearing)
			bearing =  math.floor((180/math.pi)*(bearing)/5+.5) --every 5th degree
			readings[bearing] = item[2]
	end

	local signal = 0
	local next_d = 2
	local prev_d = 0
	--Base signal 0
	--Static rand(0-20)
	--Detection 1-
	for d=1,72 do
		if d+1 > 72 then next_d = 1 else next_d = d+1 end
		if d-1 < 1 then prev_d = 72 else prev_d = d-1 end
		signal = 0
		if readings[d] then
			signal = readings[d] + math.random()*3 + 3
		elseif readings[next_d] then --If there is something on the next degree, show a small signal
			signal = (readings[next_d] / 2) + math.random()*3 + 3--plus a little random
		elseif readings[prev_d] then --If there was something on the previous degree, show signal
			signal = (readings[prev_d] / 2) + math.random()*3 + 3--plus a little random
			readings[d] = nil --so that the NEXT degree wont think this is a detection
		end
		readings2[d] = signal  		
	end
	
	--Fill in the blanks with randoms
	for d=1,72 do
		if readings2[d] == 0 then readings2[d] = math.random()*5 + 2 end
	end

	return readings2
end

function hud.drawScan(readings)
	for d=1, 72 do
		love.graphics.rotate((2*math.pi)/72)
		--love.graphics.line(-2, -60, 0, -60-readings[d])
		love.graphics.line(0, -60-readings[d], 0, -60)
	end
end

function hud.targetingComputer()
	love.graphics.setColor(255,0,0)
	if hud.fighter and hud.fighter.targetedObject and hud.fighter.detectedObjects[hud.fighter.targetedObject] then
		local aim_point_x, aim_point_y = 0,0
		
		local targeted_object = hud.fighter.detectedObjects[hud.fighter.targetedObject][1]
		local distance = distanceToXY(hud.fighter.x,hud.fighter.y,targeted_object.x,targeted_object.y)
		aim_point_x = targeted_object.x+((targeted_object.xVel-hud.fighter.xVel)*(distance/500)) --500 is bullet velocity. FUCK LUA AND NO CONSTANTS
		aim_point_y = -targeted_object.y-((targeted_object.yVel-hud.fighter.yVel)*(distance/500))

		love.graphics.circle("line",targeted_object.x,-targeted_object.y,targeted_object.boundingRadius, 10)
		love.graphics.circle("line",aim_point_x, aim_point_y, 20, 10)
		love.graphics.point(aim_point_x,aim_point_y)
		love.graphics.line(targeted_object.x,-targeted_object.y, aim_point_x, aim_point_y)
	end
end
