require "WorldObject"
require "ProximitySensor"

Missile = inheritsFrom(WorldObject)
function Missile:new(world, from_fighter, x, y, xVel, yVel, heading, rVel, delay)
	local self = self:create()
	--"from_fighter" currently unused. Probably use it for scorekeeping or something
	self.name = "Missile"
	self.x = x
	self.y = y
	self.heading = heading
	self.rVel = rVel
	self.mass = .1

	self.fromFighter = from_fighter
	self.xVel = xVel + (math.sin(heading)*50)
	self.yVel = yVel + (math.cos(heading)*50)
	self.armed = false
	self.ignited = false
	local delay = delay or 3
	self.ignitionTime = love.timer.getTime() + delay
	self.armTime = self.ignitionTime + 3

	self.status = 1

	self.image_map = nil
	self.burn_quad = nil
	self.missile_quad = nil
	self.burn_animation_timer = 0
	self.offset_x = 0
	self.offset_y = 0

	self.detectedObjects = nil
	self.bestTarget = nil
	self.world = world
	local trail = SmokeTrail:new(self, self.x, self.y)
	world:addObject(trail, false,true)

	self.proximity_callback = function(world_object)
		if world_object ~= self.fromFighter and self.armed then self:explode() end
	end
	self.proximitySensor = ProximitySensor:new(self.proximity_callback, self.x, self.y, 500, Fighter)
	world:addObject(self.proximitySensor, false)


	self:loadSprite()
	return self
end



function Missile.preLoad()
	global.missileImage = love.graphics.newImage(global.missileImageName)
end
function Missile:loadSprite()
	self.image_map = global.missileImage

    local image_map_w, image_map_h = self.image_map:getWidth(), self.image_map:getHeight()
	local burn_sprite_w, burn_sprite_h = 28,126
	local burn_sprite_left, burn_sprite_top = 7, 0
	self.burn_quad = love.graphics.newQuad(burn_sprite_left, burn_sprite_top,
											burn_sprite_w, burn_sprite_h,
											image_map_w, image_map_h)
	self.missile_quad = love.graphics.newQuad(0,0,5,28,image_map_w,image_map_h)

end
function Missile:update(dt)

	self.x = self.x + self.xVel*dt
	self.y = self.y + self.yVel*dt

	self.proximitySensor.x = self.x
	self.proximitySensor.y = self.y  - (math.cos(self.heading)*250)

	if self.ignited or love.timer.getTime() > self.ignitionTime then
		self.ignited = true
		self.burn_animation_timer = self.burn_animation_timer + dt
		if self.burn_animation_timer > 0.02 then 
			self:scan()
			self.offset_x = (self.offset_x + 28) % (4*28)
			self.burn_quad:setViewport(7+self.offset_x, 0, 26, 126)
			self.burn_animation_timer = 0
		end
		if self.bestTarget then
			local angle = XYToAngle(self.x, self.y, self.bestTarget.x+self.bestTarget.xVel, self.bestTarget.y+self.bestTarget.yVel)
			print ("Target Angle: " .. angle)
			if rationalizeRad(angle - self.heading) > 3.14 then
				self.heading = self.heading -.3*dt
				print ("Target left")
			else
				self.heading = self.heading +.3*dt
				print ("Target right")
			end
			print ("Target relative: " .. rationalizeRad(angle - self.heading))
		end
		self.xVel = self.xVel + (math.sin(self.heading)*1) --1 is the acceleration speed
		self.yVel = self.yVel + (math.cos(self.heading)*1)
		
	end
	if self.armed or love.timer.getTime() > self.armTime then
		self.armed = true
	end
	if self.status < 0 then
		self.proximitySensor.status = 0
	end
end

function Missile:draw()
	love.graphics.rotate(self.heading)
	love.graphics.translate(-2.5, -14)
	love.graphics.drawq(self.image_map, self.missile_quad, 0, 0, 0)
	if self.ignited then
		love.graphics.translate(-10, 20)
		love.graphics.drawq(self.image_map, self.burn_quad, 0, 0, 0)
	end
end

function Missile:receiveCollision(hit_list, world_object)
	if world_object.mass > 0 then self.status = -1 end
end

function Missile:explode()
	self.status = -2 --Explosion B
end	

function Missile:scan()
	self.bestTarget = nil
	self.bestTargetEmission = 0
	local emission = 0

	local angle = 0
	for i,world_object in pairs(self.world.energyObjects) do
		if world_object ~= self then
			distance = distanceToXY(self.x, self.y, world_object.x, world_object.y)
			emission = world_object:getEmission() / (distance / global.energyFade)			
			angle = XYToAngle(self.x, self.y, world_object.x, world_object.y)
			angle = rationalizeRad(self.heading - angle)
			if not world_object:isa(Missile) then
				if angle < .98 or angle > 5.22 then
					if self.bestTarget == nil or emission > self.bestTargetEmission then 
						self.bestTarget = world_object
						self.bestTargetEmission = emission
					end
				end					
			end
		end 	
	end
end

function Missile:getEmission()
	local emission = 0
	if self.ignited then emission = 80 end
	if self.armed then emission = emission + 20 end
	return emission
end
