require "math"
--Collision detection
--http://cgp.wikidot.com/circle-to-circle-collision-detection



--List1 is a list of hitboxes Receiving a hit from object2
--Each list is a list of circle hit boxes, each circle has x,y,r (radius
--Each circle should be the actual circle on the world map, properly translated
--and not just relative to the object
--Returns: the number of the hitCircle of Object 1 that made contact (the first circle that 
--made contact)
function circlesAreColliding(list1, list2)
	local hit_list = {} --A list of boxes on object1 that were hit by object2
	for i,circle in ipairs(list1) do --Each hit circle in object 1
		for j,circle2 in ipairs(list2) do --Each hit circle in object2
			if check2Circles(circle, circle2) then 
				hit_list[#hit_list + 1] = i
			end			
		end
	end
	return hit_list
end
function pointVsCircle(x, y, c1)
	local dx = c2.x - x
	local dy = c2.y - y
	local radii = c1.r
	if ( dx * dx )  + ( dy * dy ) < radii * radii then 
		return true
	end
	return false

end

--Two circles
--Circle: C.x, C.y, C.r
function check2Circles(c1, c2)
	--compare the distance to combined radii
	local dx = c2.x - c1.x
	local dy = c2.y - c1.y
	local radii = c1.r + c2.r
	if ( dx * dx )  + ( dy * dy ) < radii * radii then 
		return true
	end
	return false
end

--Physics function. Just enough for objects to react a bit when they touch each other.
--returns new xv1,yv1,xv2,yv2
function collideObjects(xv1,yv1,m1,xv2,yv2,m2)
	--Collision with markers etc.
	if m1 <= 0 or m2 <= 0 then 
		return xv1, yv1, xv2, yv2
	end
	-- v1  = ((m1-m2)/(m1+m2))u1 + ((2msub2)/(m1+m2))u2
	local new_xv1, new_xv2, new_yv1, new_yv2 = 0
	local ellastic_factor = .9
	new_xv2 = (((m2-m1)/(m2+m1))*xv2 + ((2*m1)/(m2+m1))*xv1) * ellastic_factor
	new_yv2 = (((m2-m1)/(m2+m1))*yv2	+ ((2*m1)/(m2+m1))*yv1) * ellastic_factor

	new_xv1 = (((m1-m2)/(m1+m2))*xv1	+ ((2*m2)/(m1+m2))*xv2) * ellastic_factor
	new_yv1 = (((m1-m2)/(m1+m2))*yv1	+ ((2*m2)/(m1+m2))*yv2) * ellastic_factor

	
	
	return new_xv1, new_yv1, new_xv2, new_yv2
end

function checkRectangles(ax1,ay1,aw,ah, bx1,by1,bw,bh)

  local ax2,ay2,bx2,by2 = ax1 + aw, ay1 + ah, bx1 + bw, by1 + bh
  return ax1 < bx2 and ax2 > bx1 and ay1 < by2 and ay2 > by1
end
