require "math"
--Only based on current velocity, not accelerations
function predictXY(world_object, after_seconds)
	local x, y = world_object.x, world_object.y
	x = x + world_object.xVel*after_seconds
	y = y + world_object.yVel*after_seconds
	return x, y
end

function XYToAngle(from_x, from_y, to_x, to_y)
	local x, y = to_x-from_x, to_y-from_y -- now they're relative
	local angle = math.atan2(x,y)
	return rationalizeRad(angle)
end 

function rationalizeRad(rad)
	rad = rad % (math.pi*2)
	if rad < 0 then rad = 2*math.pi + rad end
	return rad
end

--Probably a heavy function due to the sqrt
function distanceToXY(from_x, from_y, to_x, to_y)
	local x, y = math.abs(to_x-from_x), math.abs(to_y-from_y) -- now they're relative
	return math.sqrt(x*x + y*y)
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

--Nearest multiple (n) of size is returned
function multiple(n, size)
	size = size or 10
	return round(n/size)*size
end

function radToDeg(rad)
	return 360*rad/(2*math.pi)
end

function inheritsFrom( baseClass )

    local new_class = {}
    local class_mt = { __index = new_class }

    function new_class:create()
        local newinst = {}
        setmetatable( newinst, class_mt )
        return newinst
    end

    if nil ~= baseClass then
        setmetatable( new_class, { __index = baseClass } )
    end

    -- Implementation of additional OO properties starts here --

    -- Return the class object of the instance
    function new_class:class()
        return new_class
    end

    -- Return the super class object of the instance
    function new_class:superClass()
        return baseClass
    end

    -- Return true if the caller is an instance of theClass
    function new_class:isa( theClass )
        local b_isa = false

        local cur_class = new_class

        while ( nil ~= cur_class ) and ( false == b_isa ) do
            if cur_class == theClass then
                b_isa = true
            else
                cur_class = cur_class:superClass()
            end
        end

        return b_isa
    end

    return new_class
end


